@@ -1,6 +1,6 @@
 /*
- * search-tool.c
- * Copyright 2011-2012 John Lindgren
+ * search-tool.cc
+ * Copyright 2011-2015 John Lindgren
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -20,13 +20,16 @@
 #include <string.h>
 #include <gtk/gtk.h>
 
+#define AUD_PLUGIN_GLIB_ONLY
 #include <libaudcore/audstrings.h>
 #include <libaudcore/hook.h>
 #include <libaudcore/i18n.h>
 #include <libaudcore/playlist.h>
 #include <libaudcore/plugin.h>
+#include <libaudcore/mainloop.h>
 #include <libaudcore/multihash.h>
 #include <libaudcore/runtime.h>
+#include <libaudgui/libaudgui-gtk.h>
 #include <libaudgui/list.h>
 #include <libaudgui/menu.h>
 
@@ -101,8 +104,9 @@ static Index<const Item *> items;
 static int hidden_items;
 static Index<bool> selection;
 
-static gboolean adding;
-static int search_source;
+static bool adding;
+static QueuedFunc search_timer;
+static bool search_pending;
 
 static GtkWidget * entry, * help_label, * wait_label, * scrolled, * results_list, * stats_label;
 
@@ -127,7 +131,7 @@ static int create_playlist ()
     return list;
 }
 
-static int get_playlist (gboolean require_added, gboolean require_scanned)
+static int get_playlist (bool require_added, bool require_scanned)
 {
     if (playlist_id < 0)
         return -1;
@@ -148,11 +152,6 @@ static int get_playlist (gboolean require_added, gboolean require_scanned)
     return list;
 }
 
-static void set_search_phrase (const char * phrase)
-{
-    search_terms = str_list_to_index (str_tolower_utf8 (phrase), " ");
-}
-
 static String get_path ()
 {
     String path1 = aud_get_str ("search-tool", "path");
@@ -182,7 +181,7 @@ static void create_database (int list)
 
     for (int e = 0; e < entries; e ++)
     {
-        Tuple tuple = aud_playlist_entry_get_tuple (list, e, Playlist::Guess);
+        Tuple tuple = aud_playlist_entry_get_tuple (list, e, Playlist::Nothing);
 
         aud::array<SearchField, String> fields;
         fields[SearchField::Genre] = tuple.get_str (Tuple::Genre);
@@ -383,7 +382,7 @@ static void show_hide_widgets ()
     }
 }
 
-static int search_timeout (void * unused = nullptr)
+static void search_timeout (void * = nullptr)
 {
     do_search ();
 
@@ -403,21 +402,8 @@ static int search_timeout (void * unused = nullptr)
 
     gtk_label_set_text ((GtkLabel *) stats_label, stats);
 
-    if (search_source)
-    {
-        g_source_remove (search_source);
-        search_source = 0;
-    }
-
-    return false;
-}
-
-static void schedule_search ()
-{
-    if (search_source)
-        g_source_remove (search_source);
-
-    search_source = g_timeout_add (SEARCH_DELAY, search_timeout, nullptr);
+    search_timer.stop ();
+    search_pending = false;
 }
 
 static void update_database ()
@@ -495,11 +481,8 @@ static void search_cleanup ()
     hook_dissociate ("playlist scan complete", scan_complete_cb);
     hook_dissociate ("playlist update", playlist_update_cb);
 
-    if (search_source)
-    {
-        g_source_remove (search_source);
-        search_source = 0;
-    }
+    search_timer.stop ();
+    search_pending = false;
 
     search_terms.clear ();
     items.clear ();
@@ -509,9 +492,9 @@ static void search_cleanup ()
     destroy_database ();
 }
 
-static void do_add (gboolean play, String & title)
+static void do_add (bool play, bool set_title)
 {
-    if (search_source)
+    if (search_pending)
         search_timeout ();
 
     int list = aud_playlist_by_unique_id (playlist_id);
@@ -519,6 +502,7 @@ static void do_add (gboolean play, String & title)
     int n_selected = 0;
 
     Index<PlaylistAddItem> add;
+    String title;
 
     for (int i = 0; i < n_items; i ++)
     {
@@ -531,7 +515,7 @@ static void do_add (gboolean play, String & title)
         {
             add.append (
                 aud_playlist_entry_get_filename (list, entry),
-                aud_playlist_entry_get_tuple (list, entry, Playlist::Guess)
+                aud_playlist_entry_get_tuple (list, entry, Playlist::Nothing)
             );
         }
 
@@ -540,10 +524,11 @@ static void do_add (gboolean play, String & title)
             title = item->name;
     }
 
-    if (n_selected != 1)
-        title = String ();
+    int list2 = aud_playlist_get_active ();
+    aud_playlist_entry_insert_batch (list2, -1, std::move (add), play);
 
-    aud_playlist_entry_insert_batch (aud_playlist_get_active (), -1, std::move (add), play);
+    if (set_title && n_selected == 1)
+        aud_playlist_set_title (list2, title);
 }
 
 static void action_play ()
@@ -556,29 +541,19 @@ static void action_play ()
     else
         aud_playlist_queue_delete (list, 0, aud_playlist_queue_count (list));
 
-    String title;
-    do_add (true, title);
+    do_add (true, false);
 }
 
 static void action_create_playlist ()
 {
-    aud_playlist_insert (-1);
-    aud_playlist_set_active (aud_playlist_count () - 1);
-
-    String title;
-    do_add (false, title);
-
-    if (title)
-        aud_playlist_set_title (aud_playlist_count () - 1, title);
+    aud_playlist_new ();
+    do_add (false, true);
 }
 
 static void action_add_to_playlist ()
 {
-    if (aud_playlist_by_unique_id (playlist_id) == aud_playlist_get_active ())
-        return;
-
-    String title;
-    do_add (false, title);
+    if (aud_playlist_by_unique_id (playlist_id) != aud_playlist_get_active ())
+        do_add (false, false);
 }
 
 static void list_get_value (void * user, int row, int column, GValue * value)
@@ -662,8 +637,10 @@ static const AudguiListCallbacks list_callbacks = {
 
 static void entry_cb (GtkEntry * entry, void * unused)
 {
-    set_search_phrase (gtk_entry_get_text ((GtkEntry *) entry));
-    schedule_search ();
+    const char * text = gtk_entry_get_text ((GtkEntry *) entry);
+    search_terms = str_list_to_index (str_tolower_utf8 (text), " ");
+    search_timer.queue (SEARCH_DELAY, search_timeout, nullptr);
+    search_pending = true;
 }
 
 static void refresh_cb (GtkButton * button, GtkWidget * chooser)
@@ -677,52 +654,53 @@ static void refresh_cb (GtkButton * button, GtkWidget * chooser)
 
 void * SearchTool::get_gtk_widget ()
 {
-    GtkWidget * vbox = gtk_vbox_new (FALSE, 6);
+    GtkWidget * vbox = gtk_vbox_new (false, 6);
 
     entry = gtk_entry_new ();
     gtk_entry_set_icon_from_icon_name ((GtkEntry *) entry, GTK_ENTRY_ICON_PRIMARY, "edit-find");
     (void) _("Search library");  // translated string is used in GTK3 branch
     g_signal_connect (entry, "destroy", (GCallback) gtk_widget_destroyed, & entry);
-    gtk_box_pack_start ((GtkBox *) vbox, entry, FALSE, FALSE, 0);
+    gtk_box_pack_start ((GtkBox *) vbox, entry, false, false, 0);
 
     help_label = gtk_label_new (_("To import your music library into "
      "Audacious, choose a folder and then click the \"refresh\" icon."));
-    gtk_widget_set_size_request (help_label, 194, -1);
-    gtk_label_set_line_wrap ((GtkLabel *) help_label, TRUE);
+    int label_width = aud::rescale (audgui_get_dpi (), 4, 7);
+    gtk_widget_set_size_request (help_label, label_width, -1);
+    gtk_label_set_line_wrap ((GtkLabel *) help_label, true);
     g_signal_connect (help_label, "destroy", (GCallback) gtk_widget_destroyed, & help_label);
-    gtk_widget_set_no_show_all (help_label, TRUE);
-    gtk_box_pack_start ((GtkBox *) vbox, help_label, TRUE, FALSE, 0);
+    gtk_widget_set_no_show_all (help_label, true);
+    gtk_box_pack_start ((GtkBox *) vbox, help_label, true, false, 0);
 
     wait_label = gtk_label_new (_("Please wait ..."));
     g_signal_connect (wait_label, "destroy", (GCallback) gtk_widget_destroyed, & wait_label);
-    gtk_widget_set_no_show_all (wait_label, TRUE);
-    gtk_box_pack_start ((GtkBox *) vbox, wait_label, TRUE, FALSE, 0);
+    gtk_widget_set_no_show_all (wait_label, true);
+    gtk_box_pack_start ((GtkBox *) vbox, wait_label, true, false, 0);
 
     scrolled = gtk_scrolled_window_new (nullptr, nullptr);
     g_signal_connect (scrolled, "destroy", (GCallback) gtk_widget_destroyed, & scrolled);
     gtk_scrolled_window_set_shadow_type ((GtkScrolledWindow *) scrolled, GTK_SHADOW_IN);
     gtk_scrolled_window_set_policy ((GtkScrolledWindow *) scrolled,
      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-    gtk_widget_set_no_show_all (scrolled, TRUE);
-    gtk_box_pack_start ((GtkBox *) vbox, scrolled, TRUE, TRUE, 0);
+    gtk_widget_set_no_show_all (scrolled, true);
+    gtk_box_pack_start ((GtkBox *) vbox, scrolled, true, true, 0);
 
     results_list = audgui_list_new (& list_callbacks, nullptr, items.len ());
     g_signal_connect (results_list, "destroy", (GCallback) gtk_widget_destroyed, & results_list);
-    gtk_tree_view_set_headers_visible ((GtkTreeView *) results_list, FALSE);
+    gtk_tree_view_set_headers_visible ((GtkTreeView *) results_list, false);
     audgui_list_add_column (results_list, nullptr, 0, G_TYPE_STRING, -1);
     gtk_container_add ((GtkContainer *) scrolled, results_list);
 
     stats_label = gtk_label_new ("");
     g_signal_connect (stats_label, "destroy", (GCallback) gtk_widget_destroyed, & stats_label);
-    gtk_widget_set_no_show_all (stats_label, TRUE);
-    gtk_box_pack_start ((GtkBox *) vbox, stats_label, FALSE, FALSE, 0);
+    gtk_widget_set_no_show_all (stats_label, true);
+    gtk_box_pack_start ((GtkBox *) vbox, stats_label, false, false, 0);
 
-    GtkWidget * hbox = gtk_hbox_new (FALSE, 6);
-    gtk_box_pack_end ((GtkBox *) vbox, hbox, FALSE, FALSE, 0);
+    GtkWidget * hbox = gtk_hbox_new (false, 6);
+    gtk_box_pack_end ((GtkBox *) vbox, hbox, false, false, 0);
 
     GtkWidget * chooser = gtk_file_chooser_button_new (_("Choose Folder"),
      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
-    gtk_box_pack_start ((GtkBox *) hbox, chooser, TRUE, TRUE, 0);
+    gtk_box_pack_start ((GtkBox *) hbox, chooser, true, true, 0);
 
     gtk_file_chooser_set_filename ((GtkFileChooser *) chooser, get_path ());
 
@@ -730,7 +708,7 @@ void * SearchTool::get_gtk_widget ()
     gtk_container_add ((GtkContainer *) button, gtk_image_new_from_icon_name
      ("view-refresh", GTK_ICON_SIZE_BUTTON));
     gtk_button_set_relief ((GtkButton *) button, GTK_RELIEF_NONE);
-    gtk_box_pack_start ((GtkBox *) hbox, button, FALSE, FALSE, 0);
+    gtk_box_pack_start ((GtkBox *) hbox, button, false, false, 0);
 
     search_init ();
 
